package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderSelectDto;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderOperatingLog;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderSkuBomVo;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.mapper.order.OrderMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.OrderOperatingLogService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, OrderInfo> implements OrderService {

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderOperatingLogService orderOperatingLogService;

    @Override
    public Page<OrderInfoVo> getPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.like("d", Department::getName, dto.getDepartmentName());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getStatus, dto.getStatus());
        wrapper.eq("o", OrderInfo::getSettlementStatus, dto.getSettlementStatus());
        wrapper.ge("o", OrderInfo::getWlnCreateTime, dto.getBeginTime());
        wrapper.le("o", OrderInfo::getWlnCreateTime, dto.getEndTime());

        Page<OrderInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<OrderInfoVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        for (OrderInfoVo record : records) {
            record.setDeliveryTime(DateUtil.offsetDay(record.getCreateTime(), 1));
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        // 订单
        OrderInfoVo result = BeanUtil.toBean(orderInfo, OrderInfoVo.class);
        result.setDeliveryTime(DateUtil.offsetDay(result.getCreateTime(), 1));
        if (orderInfo.getDepartmentId() == null) {
            return result;
        }
        Department department = departmentService.getById(orderInfo.getDepartmentId());
        if (department != null) {
            result.setDepartmentName(department.getName());
        }

        // 订单明细
        List<OrderSku> tempOrderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuVo> orderSkuList = BeanUtil.copyToList(tempOrderSkuList, OrderSkuVo.class);
        skuSpecService.attributeAssign(orderSkuList, OrderSkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setBomSpecId(skuSpec.getBomSpecId());
        });
        bomSpecService.attributeAssign(orderSkuList, OrderSkuVo::getBomSpecId, (item, bomSpec) -> {
            item.setName(bomSpec.getName());
            item.setCode(bomSpec.getCode());
        });
        result.setOrderSkuList(orderSkuList);

        // 包材配件/单品
        List<OrderSkuBom> tempOrderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        List<OrderSkuBomVo> orderSkuBomList = BeanUtil.copyToList(tempOrderSkuBomList, OrderSkuBomVo.class);
        bomSpecService.attributeAssign(orderSkuBomList, OrderSkuBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecName(bomSpec.getName());
        });
        Map<Long, List<OrderSkuBomVo>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        for (OrderSkuVo orderSkuVo : orderSkuList) {
            orderSkuVo.setOrderSkuBomList(orderSkuBomMap.getOrDefault(orderSkuVo.getId(), Collections.emptyList()));
        }

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(OrderInfoDto orderDto) {
        this.save(orderDto);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(OrderInfoDto orderDto) {
        this.updateById(orderDto);
        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(orderDto.getUpdateType());
        orderOperatingLog.setOrderId(orderDto.getId());
        orderOperatingLog.setOrderCode(orderDto.getCode());
        orderOperatingLogService.save(orderOperatingLog);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        OrderInfo orderInfo = getById(id);

        this.removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));


        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(30);
        orderOperatingLog.setOrderId(id);
        orderOperatingLog.setOrderCode(orderInfo.getCode());
        orderOperatingLogService.save(orderOperatingLog);
    }

}
